package org.kantega.jexmec.plexus;

/*
 * Copyright 2009 Kantega AS
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.kantega.jexmec.*;
import org.codehaus.plexus.DefaultPlexusContainer;
import org.codehaus.plexus.PlexusContainerException;
import org.codehaus.plexus.DefaultComponentLookupManager;
import org.codehaus.plexus.configuration.PlexusConfigurationException;
import org.codehaus.plexus.classworlds.realm.ClassRealm;
import org.codehaus.plexus.component.repository.exception.ComponentLookupException;
import org.codehaus.plexus.component.repository.exception.ComponentRepositoryException;
import org.codehaus.plexus.component.repository.ComponentDescriptor;

import java.util.*;

/**
 * Implementation of {@link PluginLoader} based on the <a href="http://plexus.codehaus.org/">Plexus</a> IoC container.
 */
public class PlexusPluginLoader<P extends Plugin> implements PluginLoader<P> {
    private DefaultPlexusContainer container;

    private volatile int i = 0;

    public PlexusPluginLoader() {

    }

    public void unloadPlugins(ClassLoader classLoader) {
    }

    public void start() {
        try {
            container = new DefaultPlexusContainer();
        } catch (PlexusContainerException e) {
            throw new RuntimeException(e);
        }
    }

    public void stop() {
        container.dispose();
        container = null;
    }


    public List<P> loadPlugins(Class<P> pluginClass, ClassLoader classLOader, final ServiceLocator serviceLocator) {


        final Map<String, Object> servicesByName = new HashMap<String, Object>();

        for (ServiceKey serviceKey : serviceLocator.getServiceKeys()) {
            String name = serviceKey.getServiceName() == null ? serviceKey.getClass().getName() : serviceKey.getServiceName();

            @SuppressWarnings("unchecked")
            Object implementation = serviceLocator.lookupService(serviceKey.getServiceClass(), serviceKey.getServiceName());
            servicesByName.put(name, implementation);
        }

        final DefaultComponentLookupManager lookup = new DefaultComponentLookupManager() {
            @Override
            public Object lookup(String componentRole, String roleHint, ClassRealm realm) throws ComponentLookupException {
                if (servicesByName.containsKey(componentRole)) {
                    return servicesByName.get(componentRole);
                } else {
                    return super.lookup(componentRole, roleHint, realm);
                }

            }
        };

        try {

            lookup.setContainer(container);
            container.setComponentLookupManager(lookup);



            final ClassRealm classRealm = new ClassRealm(container.getClassWorld(), "pluginClassLoader" + (i++), classLOader);


            List<P> plugins = new ArrayList<P>();

            for (Object obj : container.discoverComponents(classRealm)) {
                container.addComponentDescriptor((ComponentDescriptor) obj);
                container.setLookupRealm(classRealm);

                @SuppressWarnings("unchecked")
                List<P> lookedUpPlugins = container.lookupList(pluginClass);
                plugins.addAll(lookedUpPlugins);
            }

            return plugins;


        } catch (ComponentLookupException e) {
            throw new RuntimeException(e);
        } catch (ComponentRepositoryException e) {
            throw new RuntimeException(e);
        } catch (PlexusConfigurationException e) {
            throw new RuntimeException(e);
        }
    }

}
